"""
Copyright 2008-2014 by Aerospike, Inc. All rights reserved.
THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE. THE COPYRIGHT NOTICE
ABOVE DOES NOT EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION.
"""


# -*- coding: utf-8 -*-
from alerts import Alerts
from helpers.info_dict import Integer


class NamespaceStats(Alerts):

    def __init__(self, cluster_id, node_address, namespace_name):

        Alerts.__init__(self, cluster_id)
        self._node_address = node_address
        self._namespace_name = namespace_name

        self._stats = {}
        self.attr_to_handler = (
            ('available_pct', 'on_available_pct_change'),
            ('free-pct-disk', 'on_used_disk_pct_change_stop_writes'),
            ('free-pct-memory', 'on_used_memory_pct_change_stop_writes'),
            ('free-pct-disk', 'on_used_disk_pct_change_high_water_mark'),
            ('free-pct-memory', 'on_used_memory_pct_change_high_water_mark')
        )

        self.available_pct_alert_data = dict(status="green", last_id=None)
        self.used_disk_pct_stop_writes_alert_data = dict(status="green", last_id=None)
        self.used_memory_pct_stop_writes_alert_data = dict(status="green", last_id=None)
        self.used_disk_pct_high_water_alert_data = dict(status="green", last_id=None)
        self.used_memory_pct_high_water_alert_data = dict(status="green", last_id=None)

    def on_available_pct_change(self):

        messages = dict(
            red="Contiguous Disk space available for new writes on namespace <strong>%s on %s</strong> below 10%%",
            yellow="Contiguous Disk space available for new writes on namespace <strong>%s on %s</strong> below 20%%",
            green="Contiguous Disk space available for new writes on namespace <strong>%s on %s</strong> above 20%% now"
        )

        available_pct = self._stats.get_as_int('available_pct')
        if available_pct is None:
            return

        alert_status = self.get_alert_status(
            available_pct.is_less_than(10),  # Red alert condition
            available_pct.is_less_than(20)   # Yellow alert condition
        )
        if alert_status != self.available_pct_alert_data['status']:
            self.available_pct_alert_data['status'] = alert_status
            self.available_pct_alert_data['last_id'] = self.create_alert(
                messages[alert_status] % (self._namespace_name, self._node_address),
                alert_status, self._node_address, "alert", self.available_pct_alert_data['last_id'])

    def on_used_disk_pct_change_high_water_mark(self):

        messages = dict(
            yellow="Used disk space for namespace <strong>%s on %s</strong> above high water mark",
            green="Used disk space for namespace <strong>%s on %s</strong> below high water mark now",
        )

        free_disk = self._stats.get_as_int('free-pct-disk')
        high_water = self._stats.get_as_int('high-water-disk-pct')
        if free_disk is None or high_water is None:
            return
        used_disk = Integer(100 - free_disk)
        alert_status = self.get_alert_status(
            None,                                  # Red alert condition
            used_disk.is_greater_than(high_water)  # Yellow alert condition
        )

        if alert_status != self.used_disk_pct_high_water_alert_data['status']:
            self.used_disk_pct_high_water_alert_data['status'] = alert_status
            self.used_disk_pct_high_water_alert_data['last_id'] = self.create_alert(
                messages[alert_status] % (self._namespace_name, self._node_address),
                alert_status, self._node_address, "alert", self.used_disk_pct_high_water_alert_data['last_id'])

    def on_used_disk_pct_change_stop_writes(self):

        messages = dict(
            red="Used disk space for namespace <strong>%s on %s</strong> above stop writes limit",
            green="Used disk space for namespace <strong>%s on %s</strong> below stop writes limit now",
        )

        free_disk = self._stats.get_as_int('free-pct-disk')
        stop_writes = self._stats.get_as_int('stop-writes-pct')
        if free_disk is None or stop_writes is None:
            return
        used_disk = Integer(100 - free_disk)

        alert_status = self.get_alert_status(
            used_disk.is_greater_than(stop_writes),  # Red alert condition
            None                                     # Yellow alert condition
        )

        if alert_status != self.used_disk_pct_stop_writes_alert_data['status']:
            self.used_disk_pct_stop_writes_alert_data['status'] = alert_status
            self.used_disk_pct_stop_writes_alert_data['last_id'] = self.create_alert(
                messages[alert_status] % (self._namespace_name, self._node_address)
                , alert_status, self._node_address, "alert", self.used_disk_pct_stop_writes_alert_data['last_id'])

    def on_used_memory_pct_change_stop_writes(self):

        messages = dict(
            red="Used memory space for namespace <strong>%s on %s</strong> above stop writes limit",
            green="Used memory space for namespace <strong>%s on %s</strong> below stop writes limit now",
        )

        free_memory = self._stats.get_as_int('free-pct-memory')
        stop_writes = self._stats.get_as_int('stop-writes-pct')
        if free_memory is None or stop_writes is None:
            return
        used_memory = Integer(100 - free_memory)

        alert_status = self.get_alert_status(
            used_memory.is_greater_than(stop_writes),  # Red alert condition
            None                                       # Yellow alert condition
        )
        if alert_status != self.used_memory_pct_stop_writes_alert_data['status']:
            self.used_memory_pct_stop_writes_alert_data['status'] = alert_status
            self.used_memory_pct_stop_writes_alert_data['last_id'] = self.create_alert(
                messages[alert_status] % (self._namespace_name, self._node_address)
                , alert_status, self._node_address, "alert", self.used_memory_pct_stop_writes_alert_data['last_id'])

    def on_used_memory_pct_change_high_water_mark(self):

        messages = dict(
            yellow="Used memory space for namespace <strong>%s on %s</strong> above high water mark",
            green="Used memory space for namespace <strong>%s on %s</strong> below high water mark now",
        )

        free_memory = self._stats.get_as_int('free-pct-memory')
        high_water = self._stats.get_as_int('high-water-memory-pct')
        if free_memory is None or high_water is None:
            return
        used_memory = Integer(100 - free_memory)

        alert_status = self.get_alert_status(
            None,                                    # Red alert condition
            used_memory.is_greater_than(high_water)  # Yellow alert condition
        )
        if alert_status != self.used_memory_pct_high_water_alert_data['status']:
            self.used_memory_pct_high_water_alert_data['status'] = alert_status
            self.used_memory_pct_high_water_alert_data['last_id'] = self.create_alert(
                messages[alert_status] % (self._namespace_name, self._node_address)
                , alert_status, self._node_address, "alert", self.used_memory_pct_high_water_alert_data['last_id'])


